import EventEmitter from "events";

import { ListenerRss, ListenerRSSInfos } from "listener-rss";
import { SqliteTools } from "./sqlite-tools";

/**
 * Permit to manage a ListenerRSS array, data storage and event aggregation
 */
export class ManageListener extends EventEmitter {
  listenerArray: ListenerRss[] = [];
  looprunning: boolean = false;

  sqliteDb: SqliteTools;

  constructor(path: string) {
    super();
    this.sqliteDb = new SqliteTools(path);
  }

  async load() {
    await this.sqliteDb.ensureTableExists();
    const configs: ListenerRSSInfos[] = await this.sqliteDb.fetchAll();

    configs.forEach((config) => {
      const newListener = new ListenerRss(config);
      this.listenerArray.push(newListener);
      this.settingEvents(newListener);
    });
  }

  settingEvents(newListener: ListenerRss): void {
    newListener.on("update", (obj) => this.emit("update", obj));
    newListener.on("newEntries", (obj) => this.emit("newEntries", obj));
    newListener.on("error", (err) => this.emit("error", err));
  }

  async addNewListener(info: ListenerRSSInfos) {
    const newListener = new ListenerRss(info);
    this.listenerArray.push(newListener);
    this.settingEvents(newListener);

    await this.sqliteDb.insertListener(newListener);
  }

  async save() {
    await this.sqliteDb.updateAll(
      this.listenerArray.map((listener) => listener.getProperty())
    );
  }

  startAll(): void {
    if (!this.looprunning) {
      this.looprunning = true;
      this.listenerArray.forEach((listener) => listener.start());
    }
  }

  stopAll(): void {
    if (this.looprunning) {
      this.looprunning = false;
      this.listenerArray.forEach((listener) => listener.stop());
    }
  }
}
