import { ListenerRSSInfos } from "listener-rss";

import connect, { sql } from "@databases/sqlite";

export class SqliteTools {
  path?: string;

  constructor(path?: string) {
    this.path = path;
  }

  async ensureTableExists() {
    const db = connect(this.path);

    let req = sql`CREATE TABLE IF NOT EXISTS listeners
                  (
                    address TEXT NOT NULL UNIQUE,
                    customfields TEXT DEFAULT '[]' NOT NULL,
                    timeloop INTEGER DEFAULT 300 NOT NULL,
                    last_entries_links TEXT DEFAULT '[]' NOT NULL,
                    PRIMARY KEY (address),
                    CHECK(timeloop >= 0)
                  );`;

    await db.query(req);
    await db.dispose();
  }

  async fetchAll(): Promise<ListenerRSSInfos[]> {
    const db = connect(this.path);

    let req = sql`SELECT *
                  FROM listeners`;
    const rows = await db.query(req);
    await db.dispose();
    return rows.map((row: any) => ({
      address: row["address"],
      customfields: JSON.parse(row["customfields"]),
      timeloop: row["timeloop"],
      lastEntriesLinks: JSON.parse(row["last_entries_links"]),
    }));
  }

  async insertListener(listener: ListenerRSSInfos) {
    const db = connect(this.path);

    let req = sql`INSERT INTO listeners (address, timeloop, customfields, last_entries_links)
                  VALUES (${listener.address},
                    ${listener.timeloop},
                    ${JSON.stringify(listener.customfields ? listener.customfields : [])},
                    ${JSON.stringify(listener.lastEntriesLinks ? listener.lastEntriesLinks : [])})`;
    await db.query(req);
    await db.dispose();
  }

  async updateAll(listeners: ListenerRSSInfos[]) {
    const db = connect(this.path);

    await db.tx(async (transaction) => {
      for (const listener of listeners) {
        let req = sql`UPDATE listeners
                    SET last_entries_links = ${JSON.stringify(
                      listener.lastEntriesLinks
                    )}
                    WHERE address = ${listener.address}`;
        await transaction.query(req);
      }
    });
    await db.dispose();
  }
}
